/*
    duu.c - Disk Usage Utility, version 1.1
    Copyright 1997-1999 by Richard Dawe

    ---

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* Include files */
#include <dos.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

int main (void)
{
    struct diskfree_t disk_info;        /* Disk info - clusters, etc. */
    unsigned long disk_free;            /* Disk free space            */
    unsigned long disk_total;           /* Disk total space           */
    int disk_used_percent;              /* Percentage free disk space */
    char percent_bar[21];               /* Percentage bar             */
    unsigned long alldisks_free = 0;    /* Total free space           */
    unsigned long alldisks_total = 0;   /* Total space                */
    int alldisks_used_percent;
    int i, j;                           /* Loop vars                  */

    /* Print banner */
    printf ("DUU - Disk Usage Utility, Version 1.1\n"
            "Copyright 1997-1999 by Richard Dawe\n\n");

    /* Get free space for all drives except floppies (A, B) */
    for (i = 3; i <= 26; i++) {
        /* Check disk actually exists */
        if (!_dos_getdiskfree (i, &disk_info)) {
            /* Calculate free space as a percentage */
            disk_total =   (unsigned long) disk_info.total_clusters
                         * (unsigned long) disk_info.bytes_per_sector
                         * (unsigned long) disk_info.sectors_per_cluster
                         / 1024;

            alldisks_total += disk_total;

            disk_free =   (unsigned long) disk_info.avail_clusters
                        * (unsigned long) disk_info.bytes_per_sector
                        * (unsigned long) disk_info.sectors_per_cluster
                        / 1024;

            alldisks_free += disk_free;
            disk_used_percent = 100 - (int) (disk_free * 100 / disk_total);

            /* Print info */
            memset (percent_bar, ' ', 20);

            for (j = 0; j < (int) (disk_used_percent / 5); j++) {
                percent_bar[j] = 219;
            }

            percent_bar[20] = 0;

            printf ("%c:    %8lu K free (%8lu K) %s %3i%% full\n",
                    (char) (i+64),
                    disk_free, disk_total, percent_bar, disk_used_percent);
        }
    }

    /* Total drive info */
    alldisks_used_percent =   100
                            - (int) (alldisks_free * 100 / alldisks_total);

    memset (percent_bar, ' ', 20);

    for (j = 0; j < (int) (alldisks_used_percent / 5); j++) {
        percent_bar[j] = 219;
    }

    percent_bar[20] = 0;

    printf ("\nTotal %8lu K free (%8lu K) %s %3i%% full\n", alldisks_free,
            alldisks_total, percent_bar, alldisks_used_percent);

    exit(EXIT_SUCCESS);
}
